/*
 *  The Syncro Soft SRL License
 *
 *  Copyright (c) 1998-2012 Syncro Soft SRL, Romania.  All rights
 *  reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistribution of source or in binary form is allowed only with
 *  the prior written permission of Syncro Soft SRL.
 *
 *  2. Redistributions of source code must retain the above copyright
 *  notice, this list of conditions and the following disclaimer.
 *
 *  3. Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in
 *  the documentation and/or other materials provided with the
 *  distribution.
 *
 *  4. The end-user documentation included with the redistribution,
 *  if any, must include the following acknowledgment:
 *  "This product includes software developed by the
 *  Syncro Soft SRL (http://www.sync.ro/)."
 *  Alternately, this acknowledgment may appear in the software itself,
 *  if and wherever such third-party acknowledgments normally appear.
 *
 *  5. The names "Oxygen" and "Syncro Soft SRL" must
 *  not be used to endorse or promote products derived from this
 *  software without prior written permission. For written
 *  permission, please contact support@oxygenxml.com.
 *
 *  6. Products derived from this software may not be called "Oxygen",
 *  nor may "Oxygen" appear in their name, without prior written
 *  permission of the Syncro Soft SRL.
 *
 *  THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 *  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 *  OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 *  DISCLAIMED.  IN NO EVENT SHALL THE SYNCRO SOFT SRL OR
 *  ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 *  USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 *  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 *  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 *  OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 *  SUCH DAMAGE.
 */
package com.oxygenxml.editor.editors;

import java.util.List;

import org.eclipse.jface.action.IAction;
import org.eclipse.swt.widgets.CoolBar;

import ro.sync.annotations.api.API;
import ro.sync.annotations.api.APIType;
import ro.sync.annotations.api.SourceType;
import ro.sync.exml.workspace.api.editor.page.author.WSAuthorEditorPage;

/**
 * Abstract class allowed as an extension point to customize the menu and toolbar buttons added by our editors.
 * 
 * In your plugin in the plugin.xml you should reference it like:
 * 
 *<pre> 
 *  &lt;extension point="oxygen.plugin.id.actionBarContributorCustomizer">
 *     &lt;implementation class="my.package.CustomActionBarContributorCustomizer"/>;
 *    &lt;/extension>
 *</pre> 
 * 
 * @since 14.1
 * <br>
 * <br>
 * *********************************
 * <br>
 * EXPERIMENTAL - Subject to change
 * <br>
 * ********************************
 * </br>
 * <p>Please note that this API is not marked as final and it can change in one of the next versions of the application. If you have suggestions, 
 * comments about it, please let us know.</p>
 */
@API(type=APIType.EXTENDABLE, src=SourceType.PUBLIC)
public abstract class ActionBarContributorCustomizer {

  /**
   * Customize the actions before they are contributed to the main Document menu (XML or XSL, or XSD, etc).
   * 
   * @param actions The list of original actions.
   * @return The list of modified actions. 
   */
  public abstract List<IAction> customizeActionsContributedToDocumentMenu(List<IAction> actions);

  /**
   * Customize the actions before they are contributed to the main Document toolbar.
   * 
   * @param actions The list of original actions.
   * @return The list of modified actions. 
   */
  public abstract List<IAction> customizeActionsContributedToDocumentToolbar(List<IAction> actions);

  /**
   * Customize an internal coolbar for a cetain author page. This callback may be received more than once for the
   * same document so please make sure that you do not add the same action twice.
   * 
   * @param coolbar The internal coolbar after the fixed actions have been added.
   * @param authorEditorPage The current author editor page.
   */
  public abstract void customizeAuthorPageInternalCoolbar(CoolBar coolbar, WSAuthorEditorPage authorEditorPage);
}
