/*
 *  The Syncro Soft SRL License
 *
 *  Copyright (c) 1998-2007 Syncro Soft SRL, Romania.  All rights
 *  reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistribution of source or in binary form is allowed only with
 *  the prior written permission of Syncro Soft SRL.
 *
 *  2. Redistributions of source code must retain the above copyright
 *  notice, this list of conditions and the following disclaimer.
 *
 *  3. Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in
 *  the documentation and/or other materials provided with the
 *  distribution.
 *
 *  4. The end-user documentation included with the redistribution,
 *  if any, must include the following acknowledgment:
 *  "This product includes software developed by the
 *  Syncro Soft SRL (http://www.sync.ro/)."
 *  Alternately, this acknowledgment may appear in the software itself,
 *  if and wherever such third-party acknowledgments normally appear.
 *
 *  5. The names "Oxygen" and "Syncro Soft SRL" must
 *  not be used to endorse or promote products derived from this
 *  software without prior written permission. For written
 *  permission, please contact support@oxygenxml.com.
 *
 *  6. Products derived from this software may not be called "Oxygen",
 *  nor may "Oxygen" appear in their name, without prior written
 *  permission of the Syncro Soft SRL.
 *
 *  THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 *  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 *  OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 *  DISCLAIMED.  IN NO EVENT SHALL THE SYNCRO SOFT SRL OR
 *  ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 *  USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 *  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 *  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 *  OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 *  SUCH DAMAGE.
 */
package ro.sync.ecss.extensions.dita;

import java.io.IOException;
import java.net.URL;

import ro.sync.annotations.api.API;
import ro.sync.annotations.api.APIType;
import ro.sync.annotations.api.SourceType;
import ro.sync.contentcompletion.xml.SchemaManagerFilter;
import ro.sync.ecss.dita.DITAAccess;
import ro.sync.ecss.dita.reference.keyref.KeyInfo;
import ro.sync.ecss.extensions.api.AuthorAccess;
import ro.sync.ecss.extensions.api.AuthorExtensionStateListener;
import ro.sync.ecss.extensions.api.AuthorExternalObjectInsertionHandler;
import ro.sync.ecss.extensions.api.AuthorReferenceResolver;
import ro.sync.ecss.extensions.api.AuthorSchemaAwareEditingHandler;
import ro.sync.ecss.extensions.api.AuthorTableCellSpanProvider;
import ro.sync.ecss.extensions.api.AuthorTableColumnWidthProvider;
import ro.sync.ecss.extensions.api.ExtensionsBundle;
import ro.sync.ecss.extensions.api.ProfilingConditionalTextProvider;
import ro.sync.ecss.extensions.api.UniqueAttributesRecognizer;
import ro.sync.ecss.extensions.api.content.ClipboardFragmentProcessor;
import ro.sync.ecss.extensions.api.link.ElementLocatorProvider;
import ro.sync.ecss.extensions.api.link.LinkTextResolver;
import ro.sync.ecss.extensions.api.node.AuthorElement;
import ro.sync.ecss.extensions.api.node.AuthorNode;
import ro.sync.ecss.extensions.api.table.operations.AuthorTableOperationsHandler;
import ro.sync.ecss.extensions.commons.table.support.DITATableCellInfoProvider;
import ro.sync.ecss.extensions.dita.conref.DITAConRefResolver;
import ro.sync.ecss.extensions.dita.id.DITAUniqueAttributesRecognizer;
import ro.sync.ecss.extensions.dita.link.DitaLinkTextResolver;
import ro.sync.exml.workspace.api.node.customizer.XMLNodeRendererCustomizer;

/**
 * The DITA framework extensions bundle.
 */
@API(type=APIType.INTERNAL, src=SourceType.PUBLIC)
public class DITAExtensionsBundle extends ExtensionsBundle {
  /**
   * Author table operations handler for DITA framework.
   */
  private DITAAuthorTableOperationsHandler ditaAuthorTableOperationsHandler;
  /**
   * The unique attributes recognizer
   */
  private DITAUniqueAttributesRecognizer uniqueAttributesRecognizer;
  /**
   * Custom schema aware editing handler for DITA.
   */
  private DITASchemaAwareEditingHandler ditaSchemaAwareEditingHandler;
  
  /**
   * @see ro.sync.ecss.extensions.api.ExtensionsBundle#createAuthorExtensionStateListener()
   */
  @Override
  public AuthorExtensionStateListener createAuthorExtensionStateListener() {
    uniqueAttributesRecognizer = new DITAUniqueAttributesRecognizer();
    return uniqueAttributesRecognizer;
  }
  
  /**
   * @see ro.sync.ecss.extensions.api.ExtensionsBundle#getClipboardFragmentProcessor()
   */
  @Override
  public ClipboardFragmentProcessor getClipboardFragmentProcessor() {
    return uniqueAttributesRecognizer;
  }
  
  /**
   * @see ro.sync.ecss.extensions.api.ExtensionsBundle#createAuthorReferenceResolver()
   */
  @Override
  public AuthorReferenceResolver createAuthorReferenceResolver() {
    return new DITAConRefResolver();
  }
  
  /**
   * @see ro.sync.ecss.extensions.api.ExtensionsBundle#createAuthorTableCellSpanProvider()
   */
  @Override
  public AuthorTableCellSpanProvider createAuthorTableCellSpanProvider() {
    return new DITATableCellInfoProvider();
  }
  
  /**
   * @see ro.sync.ecss.extensions.api.ExtensionsBundle#createAuthorTableColumnWidthProvider()
   */
  @Override
  public AuthorTableColumnWidthProvider createAuthorTableColumnWidthProvider() {
    return new DITATableCellInfoProvider();
  }
  
  /**
   * @see ro.sync.ecss.extensions.api.Extension#getDescription()
   */
  @Override
  public String getDescription() {
    return "DITA extensions bundle implementation";
  }
  
  /**
   * @see ro.sync.ecss.extensions.api.ExtensionsBundle#getDocumentTypeID()
   */
  @Override
  public String getDocumentTypeID() {
    return "DITA.document.type";
  }
  
  /**
   * @see ro.sync.ecss.extensions.api.ExtensionsBundle#getUniqueAttributesIdentifier()
   */
  @Override
  public UniqueAttributesRecognizer getUniqueAttributesIdentifier() {
    return uniqueAttributesRecognizer;
  }
  
  /**
   * @see ro.sync.ecss.extensions.api.ExtensionsBundle#createElementLocatorProvider()
   */
  @Override
  public ElementLocatorProvider createElementLocatorProvider() {
    return new DITAElementLocatorProvider();
  }
  
  /**
   * @see ro.sync.ecss.extensions.api.ExtensionsBundle#resolveCustomHref(java.net.URL, java.lang.String, ro.sync.ecss.extensions.api.AuthorAccess)
   */
  @Override
  public URL resolveCustomHref(URL currentEditorURL, String linkHref, AuthorAccess authorAccess) throws IOException {
    URL resolvedURL = null;
    if (linkHref != null
        && (linkHref.startsWith("Keyref ") || linkHref.startsWith("KeyWordKeyref "))) {
      //This is the special case of a DITA Key ref.
      if (linkHref.startsWith("Keyref ")) {
        linkHref = linkHref.substring("Keyref ".length());
        resolvedURL = DITAAccess.resolveKeyRef(currentEditorURL, linkHref, true);
      } else {
        linkHref = linkHref.substring("KeyWordKeyref ".length());
        //Key word pointing to the map.
        KeyInfo keyInfo = DITAAccess.getKeys(currentEditorURL).get(linkHref);
        if (keyInfo != null) {
          //Prefer the definition of the key.
          resolvedURL = keyInfo.getDefinitionLocation();
        }
      }
      if (resolvedURL == null) {
        throw new IOException("Could not resolve the target for key reference \"" + linkHref + "\"");
      }
    }
    return resolvedURL;
  }
  
  /**
   * @see ro.sync.ecss.extensions.api.ExtensionsBundle#getAuthorSchemaAwareEditingHandler()
   */
  @Override
  public AuthorSchemaAwareEditingHandler getAuthorSchemaAwareEditingHandler() {
    if (ditaSchemaAwareEditingHandler == null) {
      ditaSchemaAwareEditingHandler = new DITASchemaAwareEditingHandler();
    }
    return ditaSchemaAwareEditingHandler;
  }
  
  /**
   * @see ro.sync.ecss.extensions.api.ExtensionsBundle#createSchemaManagerFilter()
   */
  @Override
  public SchemaManagerFilter createSchemaManagerFilter() {
    return new DITASchemaManagerFilter(getDocumentTypeName());
  }
  
  /**
   * @see ro.sync.ecss.extensions.api.ExtensionsBundle#createExternalObjectInsertionHandler()
   */
  @Override
  public AuthorExternalObjectInsertionHandler createExternalObjectInsertionHandler() {
    return new DITAExternalObjectInsertionHandler();
  }
  
  /**
   * @see ro.sync.ecss.extensions.api.ExtensionsBundle#isContentReference(ro.sync.ecss.extensions.api.node.AuthorNode)
   */
  @Override
  public boolean isContentReference(AuthorNode node) {
    if(node.getType() == AuthorNode.NODE_TYPE_ELEMENT) {
      AuthorElement elem = (AuthorElement) node;
      if(elem.getAttribute("conref") != null || elem.getAttribute("conkeyref") != null) {
        return true;
      }
    }
    return false;
  }
  
  /**
   * @see ro.sync.ecss.extensions.api.ExtensionsBundle#getProfilingConditionalTextProvider()
   */
  @Override
  public ProfilingConditionalTextProvider getProfilingConditionalTextProvider() {
    return new ProfilingConditionalTextProvider() {
      /**
       * @see ro.sync.ecss.extensions.api.ProfilingConditionalTextProvider#getXMLFragmentForContentProfiling(int, int, ro.sync.ecss.extensions.api.AuthorAccess)
       */
      @Override
      public String getXMLFragmentForContentProfiling(int startOffset, int endOffset,
          AuthorAccess authorAccess) {
        return "<ph/>";
      }
    };
  }
  
  /**
   * @see ro.sync.ecss.extensions.api.ExtensionsBundle#createXMLNodeCustomizer()
   */
  @Override
  public XMLNodeRendererCustomizer createXMLNodeCustomizer() {
    return new DITANodeRendererCustomizer();
  }
  
  /**
   * @see ro.sync.ecss.extensions.api.ExtensionsBundle#getAuthorTableOperationsHandler()
   */
  @Override
  public AuthorTableOperationsHandler getAuthorTableOperationsHandler() {
    if (ditaAuthorTableOperationsHandler == null) {
      ditaAuthorTableOperationsHandler = new DITAAuthorTableOperationsHandler();
    }
    return ditaAuthorTableOperationsHandler;
  }
  
  /**
   * @see ro.sync.ecss.extensions.api.ExtensionsBundle#createLinkTextResolver()
   */
  @Override
  public LinkTextResolver createLinkTextResolver() {
    return new DitaLinkTextResolver();
  }
}