/*
 *  The Syncro Soft SRL License
 *
 *  Copyright (c) 1998-2012 Syncro Soft SRL, Romania.  All rights
 *  reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistribution of source or in binary form is allowed only with
 *  the prior written permission of Syncro Soft SRL.
 *
 *  2. Redistributions of source code must retain the above copyright
 *  notice, this list of conditions and the following disclaimer.
 *
 *  3. Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in
 *  the documentation and/or other materials provided with the
 *  distribution.
 *
 *  4. The end-user documentation included with the redistribution,
 *  if any, must include the following acknowledgment:
 *  "This product includes software developed by the
 *  Syncro Soft SRL (http://www.sync.ro/)."
 *  Alternately, this acknowledgment may appear in the software itself,
 *  if and wherever such third-party acknowledgments normally appear.
 *
 *  5. The names "Oxygen" and "Syncro Soft SRL" must
 *  not be used to endorse or promote products derived from this
 *  software without prior written permission. For written
 *  permission, please contact support@oxygenxml.com.
 *
 *  6. Products derived from this software may not be called "Oxygen",
 *  nor may "Oxygen" appear in their name, without prior written
 *  permission of the Syncro Soft SRL.
 *
 *  THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 *  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 *  OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 *  DISCLAIMED.  IN NO EVENT SHALL THE SYNCRO SOFT SRL OR
 *  ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 *  USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 *  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 *  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 *  OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 *  SUCH DAMAGE.
 */
package ro.sync.ecss.extensions.xhtml;

import java.net.URL;
import java.util.HashMap;
import java.util.Map;

import org.apache.log4j.Logger;

import ro.sync.annotations.api.API;
import ro.sync.annotations.api.APIType;
import ro.sync.annotations.api.SourceType;
import ro.sync.exml.workspace.api.node.customizer.BasicRenderingInformation;
import ro.sync.exml.workspace.api.node.customizer.NodeRendererCustomizerContext;
import ro.sync.exml.workspace.api.node.customizer.XMLNodeRendererCustomizer;

/**
 * Class used to customize the way an XHTML node is rendered in the UI. 
 * A node represents an entry from Author outline, Author bread crumb,
 * Text page outline, content completion proposals window or Elements view.
 * 
 * @author alin_balasa
 * @author alina_iordache
 */
@API(type = APIType.INTERNAL, src = SourceType.PUBLIC)
public class XHTMLNodeRendererCustomizer extends XMLNodeRendererCustomizer {
  /**
   * Logger for logging.
   */
  private static final Logger logger = Logger
      .getLogger(XHTMLNodeRendererCustomizer.class.getName());
  /**
   * Mapping between name and icon path
   */
  private static final Map<String, String> nameToIconPath = new HashMap<String, String>();

  // Initialize mapping
  static {
    // anchor
    nameToIconPath.put("a", getImageURL("/images/node-customizer/ElementA16.png"));
    
    // link
    nameToIconPath.put("link", getImageURL("/images/node-customizer/ElementLink16.png"));
            
    // body
    nameToIconPath.put("body", getImageURL("/images/node-customizer/ElementBody16.png"));
            
    String boldIcon = getImageURL("/images/node-customizer/ElementBold16.png");
        
    // bold
    nameToIconPath.put("b", boldIcon);
    
    // strong
    nameToIconPath.put("strong", boldIcon);
    
    // div
    nameToIconPath.put("div", getImageURL("/images/node-customizer/ElementDiv16.png"));
            
    // heading1
    nameToIconPath.put("h1", getImageURL("/images/node-customizer/ElementH116.png"));
            
    // heading2
    nameToIconPath.put("h2", getImageURL("/images/node-customizer/ElementH216.png"));
            
    // heading3
    nameToIconPath.put("h3", getImageURL("/images/node-customizer/ElementH316.png"));
            
    // heading4
    nameToIconPath.put("h4", getImageURL("/images/node-customizer/ElementH416.png"));
            
    // heading5
    nameToIconPath.put("h5", getImageURL("/images/node-customizer/ElementH516.png"));
            
    // heading6
    nameToIconPath.put("h6", getImageURL("/images/node-customizer/ElementH616.png"));
            
    // head
    nameToIconPath.put("head", getImageURL("/images/node-customizer/ElementHead16.png"));
            
    // html
    nameToIconPath.put("html", getImageURL("/images/node-customizer/ElementHtml16.png"));
            
    String italicIcon = getImageURL("/images/node-customizer/ElementItalic16.png");
        
    // italic
    nameToIconPath.put("i", italicIcon);
    
    // emphasize
    nameToIconPath.put("em", italicIcon);

    // image
    nameToIconPath.put("img", getImageURL("/images/node-customizer/ElementImage16.png"));
            
    // list item
    nameToIconPath.put("li", getImageURL("/images/node-customizer/ElementLi16.png"));
            
    // ordered list
    nameToIconPath.put("ol", getImageURL("/images/node-customizer/ElementOl16.png"));
            
    // paragraph
    nameToIconPath.put("p", getImageURL("/images/node-customizer/ElementPara16.png"));
            
    // span
    nameToIconPath.put("span", getImageURL("/images/node-customizer/ElementSpan16.png"));
            
    // table
    nameToIconPath.put("table", getImageURL("/images/node-customizer/ElementTable16.png"));
            
    // table body
    nameToIconPath.put("tbody", getImageURL("/images/node-customizer/ElementTBody16.png"));
            
    // table cell
    nameToIconPath.put("td", getImageURL("/images/node-customizer/ElementTd16.png"));
            
    // table foot
    nameToIconPath.put("tfoot", getImageURL("/images/node-customizer/ElementTFoot16.png"));
            
    // table head
    nameToIconPath.put("thead", getImageURL("/images/node-customizer/ElementTHead16.png"));
            
    // title
    nameToIconPath.put("title", getImageURL("/images/node-customizer/ElementTitle16.png"));
            
    // table row
    nameToIconPath.put("tr", getImageURL("/images/node-customizer/ElementTr16.png"));
    
    // th
    nameToIconPath.put("th", getImageURL("/images/node-customizer/ElementTh16.png"));
            
    // unordered list
    nameToIconPath.put("ul", getImageURL("/images/node-customizer/ElementUl16.png"));
            
    // underline
    nameToIconPath.put("u", getImageURL("/images/node-customizer/ElementUnderline16.png"));
    
    // hr
    nameToIconPath.put("hr", getImageURL("/images/node-customizer/ElementHr16.png"));

    // meta
    nameToIconPath.put("meta", getImageURL("/images/node-customizer/ElementMeta16.png"));

    // script
    nameToIconPath.put("script", getImageURL("/images/node-customizer/ElementScript16.png"));

    // style
    nameToIconPath.put("style", getImageURL("/images/node-customizer/ElementStyle16.png"));
    
    // object
    nameToIconPath.put("object", getImageURL("/images/node-customizer/ElementMedia16.png"));
    
    // audio
    nameToIconPath.put("audio", getImageURL("/images/node-customizer/ElementAudio16.png"));
    
    // video
    nameToIconPath.put("video", getImageURL("/images/node-customizer/ElementVideo16.png"));
  }

  /**
   * Searches the class-path for the image with the given path 
   * and if found it returns the string representation of its URL, otherwise it returns <code>null</code>.  
   * 
   * @param path The image path to search for.
   * 
   * @return The string representation of the image URL or <code>null</code> if the image is not found.
   */
  private static String getImageURL(String path) {
    URL imageURL = XHTMLNodeRendererCustomizer.class.getResource(path);
    if (imageURL != null) {
      return imageURL.toExternalForm();
    } else {
      logger.error(XHTMLNodeRendererCustomizer.class.getName() + " - Image not found: " + path);
      return null;
    }
  }

  @Override
  public BasicRenderingInformation getRenderingInformation(NodeRendererCustomizerContext context) {
    BasicRenderingInformation renderingInfo = new BasicRenderingInformation();
    String nodeName = context.getNodeName();
    if (nodeName != null) {
      // Get icon path from map
      String iconPath = nameToIconPath.get(nodeName);
      renderingInfo.setIconPath(iconPath);
    }
    return renderingInfo;
  }

  @Override
  public String getDescription() {
    return "XHTML Node Renderer Customizer";
  }
}